from .Constants import *
from .utils import insert_profile_variable_value, get_curr_state, get_variable_value_by_id
from variable_local.variable import VariablesLocal
from variable_local.template import ReplaceFieldsWithValues
from logger_local.LoggerComponentEnum import LoggerComponentEnum
from logger_local.Logger import Logger
from circles_local_database_python.connector import Connector
from circles_local_database_python.generic_crud import GenericCRUD

object_to_insert = {
    'component_id': DIALOG_WORKFLOW_PYTHON_PACKAGE_COMPONENT_ID,
    'component_name': DIALOG_WORKFLOW_PYTHON_PACKAGE_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
    'developer_email': 'guy.n@circ.zone'
}

logger = Logger.create_logger(object=object_to_insert)

# class Variable(object):
#     def __init__(self):
#         self.name2id_dict = {}
#         self.id2name_dict = {}
#         self.next_variable_id = 1
#         for variable_id in VARIABLE_NAMES_DICT:
#             self.add(variable_id, VARIABLE_NAMES_DICT[variable_id])

#     def add(self, variable_id : int, variable_name : str):
#         try:
#             self.name2id_dict[variable_name] = variable_id
#             self.id2name_dict[variable_id] = variable_name
#             cursor.execute("""INSERT INTO variable_table(id, name) VALUES (%s, %s)""", [variable_id, variable_name])
#             connection.commit()

#         except:
#             """TODO: add error to logger"""

#     def get_variable_id(self, variable_name : str):
#         return self.name2id_dict[variable_name]

#     def get_variable_name(self, variable_id : int):
#         return self.id2name_dict[variable_id]

#     def get_variable_value_by_name(self, language: str, variable_name : str) -> str:
#         variable_id = self.get_variable_id(variable_name)
#         return get_variable_value_by_id(self, language, variable_id)


class ProfileContext (object):
    # TODO We should consider to take ProfileContact to a separate package or merge it with UserContext
    def __init__(self, profile_id):
        self.dict = {}
        self.profile_id = profile_id
        self.chosen_poll_options = {}
        self.curr_state_id = get_curr_state(self.profile_id)
        self.variables = VariablesLocal()
        self.groups = []

    def get_variable_value_by_id(self, variable_id: int) -> str:
        logger.start(object={'variable_id': variable_id})
        variable_value = self.dict[variable_id]
        logger.end(object={'variable_value': variable_value})
        return variable_value

    def save_chosen_options(self, question_asked: str, variable_id: int, chosen_numbers_list: list, list_of_options: list):
        """Saves the options chosen by the user in the multi_choice_poll action in a dict with the question as the key
            and a list of the options chosen as the value i.e: {<question asked> : [<chosen option 1>, <chosen option 2>..]}
            Also saves the chosen options in the database."""
        logger.start(object={'question_asked': question_asked, 'variable_id': variable_id, 'chosen_numbers_list': chosen_numbers_list, 'list_of_options': list_of_options})
        self.chosen_poll_options[question_asked] = [
            list_of_options[chosen_option-1] for chosen_option in chosen_numbers_list]
        variable_value_to_insert = question_asked + " "
        for chosen_option in self.chosen_poll_options[question_asked]:
            variable_value_to_insert = variable_value_to_insert + \
                str(chosen_option) + ", "
        insert_profile_variable_value(
            self.profile_id, variable_id, variable_value_to_insert, self.curr_state_id)
        logger.end()

    def get_variable_value_by_name(self, variable_name: str) -> str:
        # TODO get_variable_value_by_variable_name
        logger.start(object={'variable_name': variable_name})
        variable_value = self.get_variable_value_by_id(self.variables.get_variable_id(variable_name))
        logger.end(object={'variable_value': variable_value})
        return variable_value
    
    # TODO Should be moved to variable-value-local-python-package
    def set(self, variable_id: int, variable_value: str):
        logger.start(object={'variable_id': variable_id, 'variable_value': variable_value})
        self.dict[variable_id] = variable_value
        connection = Connector.connect('logger')
        cursor = connection.cursor(dictionary=True, buffered=True)
        # cursor.execute("""USE logger""")
         # TODO Can we replace it with GenericCRUD
        cursor.execute("""INSERT INTO logger
                        (profile_id, state_id, variable_id, variable_value_new) 
                        VALUES (%s,%s,%s,%s)""",
                       (self.profile_id, self.curr_state_id, variable_id, variable_value))
        # cursor.execute("""USE dialog_workflow""")
        connection.commit()
        logger.end()


class ProfilesDict(object):
    def __init__(self):
        self.profiles_dict = {}

    def add(self, profile: ProfileContext):
        logger.start(object={'profile': profile})
        self.profiles_dict[profile.profile_id] = profile
        logger.end()

    def get(self, profile_id: int) -> ProfileContext:
        logger.start(object={'profile_id': profile_id})
        if profile_id not in self.profiles_dict:
            logger.end()
            return None
        else:
            profile_dict = self.profiles_dict[profile_id]
            logger.end(object={'profile_dict': profile_dict})
            return profile_dict
        # return None if profile_id not in self.profiles_dict else self.profiles_dict[profile_id]


class DialogWorkflowRecord(object):
    def __init__(self, record):
        self.curr_state_id: int = record["state_id"] if "state_id" in record else None
        self.parent_state_id: int = record["parent_state_id"] if "parent_state_id" in record else None
        self.workflow_action_id: int = record["workflow_action_id"] if "workflow_action_id" in record else None
        self.lang_code = record["lang_code"] if "lang_code" in record else None
        self.parameter1 = record["parameter1"] if "parameter1" in record else None
        self.variable1_id: int = record["variable1_id"] if "variable1_id" in record else None
        self.result_logical = record["result_logical"] if "result_logical" in record else None
        self.result_figure_min: float = record["result_figure_min"] if "result_figure_min" in record else None
        self.result_figure_max: float = record["result_figure_max"] if "result_figure_max" in record else None
        self.next_state_id: int = record["next_state_id"] if "next_state_id" in record else None
        self.no_feedback_milliseconds: float = record[
            "no_feedback_milliseconds"] if "no_feedback_milliseconds" in record else None
        self.next_state_id_if_there_is_no_feedback: int = record[
            "next_state_id_if_there_is_no_feedback"] if "next_state_id_if_there_is_no_feedback" in record else None



from enum import Enum
from dotenv import load_dotenv
load_dotenv()
# TODO I'm not sure I understand we deleted out database Class and moved to mysql - It looks to me like going backward.
# from circles_local_database_python.database import database
# import mysql.connector
import os

class action_enum(Enum):
    PRESENT_AND_CHOOSE_SCRIPT = 0
    LABEL_ACTION = 1
    TEXT_MESSAGE_ACTION = 2
    QUESTION_ACTION = 3
    JUMP_ACTION = 4
    SEND_REST_API_ACTION = 5    
    ASSIGN_VARIABLE_ACTION = 6
    INCREMENT_VARIABLE_ACTION = 7
    DECREMENT_VARIABLE_ACTION = 8
    CONDITION_ACTION = 9
    MENU_ACTION = 10
    AGE_DETECTION = 11
    MULTI_CHOICE_POLL = 12
    PRESENT_CHILD_GROUPS_NAMES_BY_ID = 13
    PRESENT_GROUPS_WITH_CERTAIN_TEXT = 14
    INSERT_MISSING_DATA = 15
    
VARIABLE_NAMES_DICT = {1 : "Person Id" , 2 : "User Id", 3 : "Profile Id", 4 : "Lang Code", 
                       5 : "Name Prefix", 6 : "First Name", 7 : "Middle Name" , 
                       8 : "Last Name", 9 : "Name Suffix",  10 : "Full Name", 
                       11 : "Country", 12 : "State" , 13 : "County" , 14 : "City", 
                       15 : "Neighborhood", 16 : "Street", 17 : "House", 18 : "Suite/Apartment", 
                       19 : "Zip Code", 20 : "Post Result", 21 : "Age", 22 : "Result"}

class communication_type_enum(Enum):
    CONSOLE = 1
    WEBSOCKET = 2

# TODO: We should align the terminology between dialog-workflow and gender-detection (INTERFACE_MODE= BATCH, INTERACTIVE)
DEFAULT_COMMUNICATION_TYPE = communication_type_enum.WEBSOCKET.value
COMMUNICATION_TYPE = DEFAULT_COMMUNICATION_TYPE if os.getenv("COMMUNICATION_TYPE") == None else os.getenv("COMMUNICATION_TYPE")

DIALOG_WORKFLOW_PYTHON_PACKAGE_COMPONENT_ID = 166
DIALOG_WORKFLOW_PYTHON_PACKAGE_COMPONENT_NAME = "dialog_workflow-python-package"

# connection = mysql.connector.connect(
#         host=os.getenv("RDS_HOSTNAME"),
#         user=os.getenv("RDS_USERNAME"),
#         password=os.getenv("RDS_PASSWORD")
#         )

# # connection = database().connect_to_database()
# cursor = connection.cursor(dictionary=True, buffered=True)
# cursor.execute("""USE dialog_workflow""")



from .utils import *
from .Act import action
from .TablesAsObjects import DialogWorkflowRecord
from variable_local.variable import *
from variable_local.template import ReplaceFieldsWithValues
from logger_local.LoggerComponentEnum import LoggerComponentEnum
from logger_local.Logger import Logger
from circles_local_database_python.connector import Connector
from circles_local_database_python.generic_crud import GenericCRUD

object_to_insert = {
    'component_id': DIALOG_WORKFLOW_PYTHON_PACKAGE_COMPONENT_ID,
    'component_name': DIALOG_WORKFLOW_PYTHON_PACKAGE_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
    'developer_email': 'alon.k@circ.zone'
}

logger = Logger.create_logger(object=object_to_insert)

def get_preferred_language(profile_id: int):
    logger.start(object={'profile_id': profile_id})
    connection = Connector.connect('profile')
    cursor = connection.cursor(dictionary=True, buffered=True)
    # TODO We should get preferred language from User-Context
    # cursor.execute("""USE profile""")
    cursor.execute(
        """SELECT preferred_lang_code FROM profile_view WHERE user_id = %s""", [profile_id])
    language = (cursor.fetchone())["preferred_lang_code"]
    # cursor.execute("""USE dialog_workflow""")
    logger.end(object={'language': language})
    return language


def get_dialog_workflow_record(profile_curr_state: int, language: str):
    logger.start(object={'profile_curr_state': profile_curr_state, 'language': language})
     # TODO Change * to the fields required
    connection = Connector.connect('dialog_workflow')
    cursor = connection.cursor(dictionary=True, buffered=True)
    cursor.execute("""SELECT * FROM dialog_workflow_state_view WHERE state_id = %s AND lang_code = %s""",
                   [profile_curr_state, language])
    optional_records = cursor.fetchall()
    random_index = random.randint(0, len(optional_records)-1)
    dialog_workflow_record = DialogWorkflowRecord(optional_records[random_index])
    logger.end(object={'dialog_workflow_record': str(dialog_workflow_record)})
    return dialog_workflow_record


def post_message(profile_id:  int, incoming_message: str):
    """This function is supposed to serve as a POST request later on using REST API.
    It runs until needing input from the user, which it then sends a json to the user with the message and exits
    PARAMS: 
        1. profile_id: the profile id that sent the request
        2. incoming_message: the message he sent"""
    logger.start(object={'profile_id': profile_id, 'incoming_message': incoming_message})
    variables = VariablesLocal()
    profile_curr_state = get_curr_state(profile_id)
    language = get_preferred_language(profile_id)
    got_response = True  # This variable indicates if we must act now as we got a response from the user or as if we should send one to him
    init_action = action(incoming_message, profile_id,
                         language, profile_curr_state, variables)
    while True:
        dialog_workflow_record = get_dialog_workflow_record(
            init_action.profile_curr_state, language)
        is_state_changed, outgoing_message = init_action.act(
            dialog_workflow_record, got_response)
        if outgoing_message != None:
            logger.end(object={'outgoing_message': outgoing_message})
            return outgoing_message
        init_action.profile_curr_state = dialog_workflow_record.next_state_id if is_state_changed == False else init_action.profile_curr_state
        update_profile_curr_state_in_DB(
            init_action.profile_curr_state, profile_id)
        got_response = False



import setuptools

setuptools.setup(
     name='dialog-workflow-local',  
     version='0.0.54',
     author="Circles",
     author_email="info@circle.life",
     description="PyPI Package for dialog workflow",
     long_description="This is a package for running the dialog workflow",
     long_description_content_type="text/markdown",
     url="https://github.com/javatechy/dokr",
     packages=setuptools.find_packages(),
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: Other/Proprietary License",
         "Operating System :: OS Independent",
     ],
 )



# TODO: Let's talk about Constants and the usage in this file
from .Constants import *
import random
import json
from logger_local.LoggerComponentEnum import LoggerComponentEnum
from logger_local.Logger import Logger
from circles_local_database_python.connector import Connector
from circles_local_database_python.generic_crud import GenericCRUD

object_to_insert = {
    'component_id': DIALOG_WORKFLOW_PYTHON_PACKAGE_COMPONENT_ID,
    'component_name': DIALOG_WORKFLOW_PYTHON_PACKAGE_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
    'developer_email': 'alon.k@circ.zone'
}

logger = Logger.create_logger(object=object_to_insert)

# from circles_local_aws_s3_storage_python.AWSStorage import AwsS3Storage

# TOOD: Please include type to all parameters.
# TODO: Please include return value.
# TODO: Please add short documentation per PEP8 standard

# TODO:Let's move to Object Oriented Programming

def get_variable_value_by_id(language: str, variable_id : int) -> str:
    logger.start(object={'variable_id': variable_id})
    connection = Connector.connect('dialog_workflow')
    cursor = connection.cursor(dictionary=True, buffered=True)
    cursor.execute("""SELECT variable_value_new FROM dialog_workflow_state_history_view WHERE variable_id= %s ORDER BY timestamp DESC""", [variable_id])
    variable_value = (cursor.fetchone())["variable_value_new"]
    logger.end(object={'variable_value': variable_value})
    return variable_value

# This code is commended since it moved to variable-local:

# class replace_fields_with_values(object):
#     def __init__(self, message, language: str, variable):
#         self.message = message
#         self.language = language
#         self.variable = variable

#     def choose_option(self, message_index : int, first_param: str):
#         """ Gets a messsage, an index of the first apearance of '|' , and the first parameter of the options
#             Returns:
#             1. One of the options randomly selected
#             2. The index of one char after the next '}'"""
#         list_of_params = [first_param]
#         while(self.message[message_index] != '}'):
#             next_param = ""
#             while(self.message[message_index] != '|' and self.message[message_index] != '}'):
#                 next_param = next_param + self.message[message_index]
#                 message_index = message_index + 1
#             list_of_params.append(next_param)
#             message_index = message_index + 1 if self.message[message_index] != '}' else message_index
#         random_index = random.randint(0, len(list_of_params)-1)
#         return list_of_params[random_index], message_index + 1

#     """For the following function to work the message must not have single '{' or '}'. They should always come in pairs"""
#     def get_variable_names_and_chosen_option(self):
#         """Returns:
#             1. A list of all variable names that were inside curly braces of message
#             2. A string that's a copy of the message but without the variable names inside curly braces,
#             and a randomly chosen parameter out of each curly braces options:
#             "Hello {First Name}, how are you {feeling|doing}?" --> "Hello {}, how are you doing?" 
#         """
#         variable_names_list = []
#         message_without_variable_names = ""
#         message_index = 0
#         while message_index < len(self.message):
#             index_after_choosing_option = None
#             if(self.message[message_index] == '{'):
#                 message_index = message_index + 1
#                 next_variable_name = ""
#                 chosen_parameter = None
#                 while self.message[message_index] != '}':
#                     if self.message[message_index] == '|':
#                         chosen_parameter, index_after_choosing_option = self.choose_option(message_index+1, next_variable_name)
#                         break
#                     next_variable_name = next_variable_name + self.message[message_index]
#                     message_index = message_index + 1
#                 if chosen_parameter != None:
#                     message_without_variable_names = message_without_variable_names + chosen_parameter
#                 else:
#                     message_without_variable_names = message_without_variable_names + "{}"
#                     variable_names_list.append(next_variable_name)
#             else:
#                 message_without_variable_names = message_without_variable_names + self.message[message_index]
#             message_index = message_index + 1 if index_after_choosing_option == None else index_after_choosing_option
#         return variable_names_list, message_without_variable_names

#     def get_variable_values_and_chosen_option(self):
#         """Returns:
#             1. A list of all variable values by variable names that were inside curly braces of message 
#             2. A string that's a copy of the message but without the variable names inside curly braces
#             and a randomly chosen parameter out of each curly braces options:
#             "Hello {First Name}, how are you {feeling|doing}?" --> "Hello {}, how are you doing?"
#         """
#         variable_names_list, message_without_variable_names = self.get_variable_names_and_chosen_option()
#         variable_value_list = []
#         for variable_name in variable_names_list:
#             variable_id = self.variable.get_variable_id(variable_name)
#             variable_value = get_variable_value_by_id(self.language, variable_id)
#             variable_value_list.append(variable_value)    
#         return message_without_variable_names.format(", ".join(variable_value_list))

# TODO change the ints to types/enums/classes
def process_message(communication_type : int, action_type : int, message : str):
    """Processes message according to which communication_type is used: console or websocket
        If console then we continue the code normally.
        If websocket then we create a json out of the given message and exit(0)"""
    logger.start(object={'communication_type': communication_type,'action_type': action_type, 'message': message})
    if communication_type == communication_type_enum.CONSOLE.value:
        message = message.replace('~', '\n')
        logger.end()
        return None
    if communication_type == communication_type_enum.WEBSOCKET.value:
        message = update_json_message(action_type, message)
        logger.end(object={'message': message})
        return message

def update_json_message(action_type: int, message: str) -> json:
    """Update message to json format"""
    logger.start(object={"action_type": action_type, "message": message})
    json_message = {"message": message}
    if action_type == action_enum.AGE_DETECTION.value:
        json_message["type"] = "command"
    else:
        json_message["type"] = "text"
    json_dumps = json.dumps(json_message) 
    logger.end(object={'json_dumps': json_dumps}) 
    return json_dumps

def update_profile_curr_state_in_DB(new_state: int, profile_id):
    """This function updates the last_dialog_workflow_state_id in the profile table to the new state.
       Note that this function UPDATES the field sand doesn't INSERT into the table. """
    logger.start(object={"new_state": new_state, 'profile_id': profile_id})
    connection = Connector.connect('profile')
    cursor = connection.cursor(dictionary=True, buffered=True)
    # cursor.execute("""USE profile""")
    cursor.execute("""UPDATE profile_table SET last_dialog_workflow_state_id = %s WHERE (user_id= %s)""", [new_state, profile_id])
    # cursor.execute("""USE dialog_workflow""")
    connection.commit()
    logger.end()

# TODO Should be moved to variable-value-local-python-package
def insert_profile_variable_value(profile_id: int, variable_id : int, variable_value : str, profile_curr_state: int):
    """This function inserts a variable value into the logger table for a given profile"""
    logger.start(object={'profile_id': profile_id, 'variable_id': variable_id, 'variable_value': variable_value, 'profile_curr_state': profile_curr_state})
    # connection = Connector.connect('logger')
    # cursor = connection.cursor(dictionary=True, buffered=True)
    # cursor.execute("""USE logger""")    
    # cursor.execute("""INSERT INTO logger_table
    #                 (profile_id, state_id, variable_id, variable_value_new) 
    #                 VALUES (%s,%s,%s,%s)""", 
    #                 (profile_id, profile_curr_state, variable_id, variable_value))
    object = {'profile_id': profile_id, 'state_id': profile_curr_state, 'variable_id': variable_id, 'variable_value_new': variable_value}
    logger.info(object = object)
    # cursor.execute("""USE dialog_workflow""")
    # connection.commit()
    logger.end()

#TODO: Make sure we have good testing coverage to this function
def store_age_detection_picture(age_range: str, profile_id: int):
    """Stores the picture in Nir's storage schema, gets a storage_id and inserts into the computer_vision_storage_table"""
    # storage = AwsS3Storage(bucket_name="storage.us-east-1.dvlp1.bubblez.life", region="us-east-1")
    # storage_id = storage.upload_file("C:\\Users\\User\\OneDrive\\Circles\\age-detection-backend\\src\\alonPicture.png", "Alon's picture", "", 1)
    
    age_range_split = age_range.split('-')
     # TODO Why this is commented?
     # TODO Why we commented this? Let's uncomment
    # min_age = int(age_range_split[0][:len(age_range_split[0])-1])
    # max_age = int((age_range_split[1])[1:])
    # cursor.execute("""USE computer_vision_storage""")
    # cursor.execute("""INSERT INTO computer_vision_storage_table 
    #                 (storage_id, profile_id, min_age, max_age) VALUES (%s, %s, %s, %s)""", 
    #                [storage_id, profile_id, min_age, max_age])
    # connection.commit()

def get_curr_state(profile_id: int):
    """Returns profiles' curernt state number"""
    logger.start(object={'profile_id': profile_id})
    connection = Connector.connect('profile')
    cursor = connection.cursor(dictionary=True, buffered=True)
    # cursor.execute("""USE profile""")
    cursor.execute("""SELECT last_dialog_workflow_state_id FROM profile_view WHERE user_id = %s ORDER BY profile_id DESC""", [profile_id])
    curr_state = int((cursor.fetchone())["last_dialog_workflow_state_id"])
    # cursor.execute("""USE dialog_workflow""")
    logger.end(object={'curr_state': curr_state})
    return curr_state

def get_child_nodes_of_current_state(fields : list, table_name : str, values_from_where_to_select : list, variables_from_where_to_select : list):
    """Recieves all of the relevant information and selects the child nodes of the current state from the given table, and returns them"""
    logger.start(object={'fields': fields, 'table_name': table_name, 'values_from_where_to_select': values_from_where_to_select, 'variables_from_where_to_select': variables_from_where_to_select})
    sql_query = "SELECT " + ", ".join(fields) + " FROM " + table_name + " WHERE "
    # Add the variables and values to the SQL query
    for i in range(len(variables_from_where_to_select)):
        sql_query += variables_from_where_to_select[i] + " = %s"
        if i != len(variables_from_where_to_select) - 1:
            sql_query += " AND "
    connection = Connector.connect('dialog_workflow')
    cursor = connection.cursor(dictionary=True, buffered=True)
    cursor.execute(sql_query, values_from_where_to_select)
    child_nodes = cursor.fetchall()
    logger.end(object={'child_nodes': child_nodes})
    return child_nodes

class group(object):
    def __init__(self, parameter1):
        self.parameter1 = parameter1
            
    def get_group_childs_id_bellow_parent_id(self) -> list[int]:  
        """returns all of the childs ids below the given parent_id.
            This function gets all of the id's of records that their parent_state_id is the given id, and continues to add id's  recursively 
            until all of the records that their parent_state_id matches an id in the table."""
        logger.start()
        connection = Connector.connect('group')
        cursor = connection.cursor(dictionary=True, buffered=True)
        # cursor.execute("""USE `group`""")
        cursor.execute("""WITH RECURSIVE cte AS (
            SELECT group_view.id FROM group_view WHERE id = %s
            UNION ALL
            SELECT group_view.id FROM group_view 
            JOIN cte ON group_view.parent_group_id = cte.id
        )
        SELECT cte.id FROM cte""", [self.parameter1])  
        group_id_dict = cursor.fetchall()
        group_childs_id = [group['id'] for group in group_id_dict]
        logger.end(object={'group_childs_id': group_childs_id})
        return group_childs_id

    def get_child_group_names(self) -> list:
        """Gets all of the child title names from the ml table of the given parent_id."""
        logger.start()
        group_ids = self.get_group_childs_id_bellow_parent_id()
        group_id_string = ','.join(str(id) for id in group_ids)
        connection = Connector.connect('group')
        cursor = connection.cursor(dictionary=True, buffered=True)
        cursor.execute(f"SELECT title FROM group_ml_en_view WHERE group_id IN ({group_id_string}) ")
        group_name_dict = cursor.fetchall()
        child_group_names = [group['title'] for group in group_name_dict]
        logger.end(object={'child_group_names': child_group_names})
        return child_group_names
    
    # def get_child_group_id(self) -> list[int]:
    #     """Gets the id of all the records with the given group name"""
    #     cursor.execute("""USE `group`""")
    #     cursor.execute("""SELECT group_id from group_ml_en_view WHERE title = %s""", [self.parameter1])
    #     group_id_dict = cursor.fetchall()
    #     return [group['group_id'] for group in group_id_dict]        

def generic_menu(options: list, got_response: bool, chosen_numbers: str, choose_one_option: bool, outgoing_message: str):
    """A generic function for displaying a menu for the user.
        Returns: If not got_response: the menu options as json to send to user.
                 Otherwise returns the chosen numbers as list in int"""
    logger.start(object={'options': options, 'got_response': got_response, 'chosen_numbers': chosen_numbers, 'choose_one_option': choose_one_option, 'outgoing_message': outgoing_message})
    if not got_response:
        outgoing_message += f"Please choose EXACTLY ONE option between 1-{len(options)}:~" if choose_one_option else f"Please select your desired choices, You may select any of the numbers between 1-{len(options)} with a comma seperator between each choice:~"        
        for i, option in enumerate(options):
            outgoing_message = outgoing_message + f'{i+1}) {option}~'
            outgoing_message_json = process_message(communication_type= COMMUNICATION_TYPE, action_type= action_enum.TEXT_MESSAGE_ACTION.value, message= outgoing_message) 
        if COMMUNICATION_TYPE == communication_type_enum.WEBSOCKET.value:
            return outgoing_message_json 
    
    if COMMUNICATION_TYPE == communication_type_enum.CONSOLE.value:  
        chosen_numbers = input()
    chosen_numbers = chosen_numbers.split(',')
    generic_menu_result = [int(x)-1 for x in chosen_numbers]
    logger.end(object={'generic_menu_result': generic_menu_result})
    return generic_menu_result